#!/usr/bin/perl

# This is a LoadLeveler submit_filter program. It takes
# as stdin the user's job command file, and outputs a
# modified job command file which LL's llsubmit command
# then parses and submits to the cluster.
#
# This particular filter does 3 things:
#    1) It ensures that node_usage=not_shared
#    2) It ensures that all job steps request
#       node_topology = island.
#    3) It ensures that the preferences of all jobs of
#       the class "interactive" include  a preference for
#       "island01", that is:
#       preferences = (Island == "island01")
#
# The above 3 things are checked for each step in the job

#system("/usr/bin/env > /tmp/env.out");

$island_pref = "(Island == \"island01\")";

# We have not yet added node_usage=not_shared for this step.
$ns_added=0;

# We have not yet added node_topology=island for this step.
$topo_added=0;

# We have not yet added preferences for this step.
$pref_added=0;
$orig_pref="";

# We have not yet determined that this is an interactive step.
$is_inter=0;

while (<>) {

	$orig_line=$_;

	# We only care about lines that begin with "#@"
	# (whitespace is permitted)
	# Any line that we don't need to modify just gets
	# re-written to stdout as-is.

	if (/^[[:space:]]*#[[:space:]]*@/) {

		# We have gotten to the end of this step
		if (/^[[:space:]]*#[[:space:]]*@[[:space:]]queue/i) {

			# If we have not already set node_usage, set it now.
			if (!$ns_added) {
				print "#@ node_usage = not_shared\n";
			}

			# If we have not already set node_usage, set it now.
			if (!$topo_added) {
				print "#@ node_topology = island\n";
			}

			# If we have not already set preferences and this is
			# an interactive step, add the preferences line now.
			if (!$pref_added && $is_inter) {
				print "#@ preferences = ", $island_pref, "\n";
			}

			# If this is not an interactive step, but the step
			# did have a preference, add it back to the jcf now
			if (!$is_inter && length($orig_pref)) {
				print $orig_pref, "\n";
			}

			# If this is not an interactive step, and the step
			# did NOT have a preference, write a blank preference
			# to the JCF to avoid incorrectly inheriting the previous
			# step's preference
			if (!$is_inter && !length($orig_pref)) {
				print "#@ preferences =\n";
			}
			print "#@ queue\n";

			$ns_added=0;
			$topo_added=0;
			$pref_added=0;
			$orig_pref="";
			$is_inter=0;
		}

		# Determine if the step is of the class "interactive"
		elsif (/^[[:space:]]*#[[:space:]]*@[[:space:]]class[[:space:]]*=[[:space:]]*interactive/i) {
			$is_inter=1;
			# If we have already encountered the preferences keyword,
			# add our island preference to it and print it
			if(length($orig_pref)) {
				print $orig_pref, " && ", $island_pref, "\n";
				$pref_added=1;
			}
			# Write the class statement back to the jcf
			print $orig_line;
		}


		# This step already contains the node_usage keyword; force
		# it to be not_shared no matter what its current value
		elsif (/^[[:space:]]*#[[:space:]]*@[[:space:]]node_usage/i) {
			print "#@ node_usage = not_shared\n";
			$ns_added=1;
		}

		# This step already contains the node_topology keyword; force
		# it to be island no matter what its current value
		elsif (/^[[:space:]]*#[[:space:]]*@[[:space:]]node_topology/i) {
			print "#@ node_topology = island\n";
			$topo_added=1;
		}

		# This step already contains the preferences keyword; add
		# our preferences to the existing line using the && operator.
		elsif (/^[[:space:]]*#[[:space:]]*@[[:space:]]preferences/i) {
			chomp $orig_line;
			$orig_pref=$orig_line;
			if ($is_inter) {
				print $orig_pref, " && ", $pool_pref, "\n";
				$pref_added=1;
			}
		}

		else {
			# The line is a jcf keyword, but we don't care about it.
			print $orig_line;
		}
	} else {
		# Not a jcf keyword
		print $orig_line;
	}
}
